@@ -1,7 +1,13 @@
-use super::parser::{resolve_flag, Cmd, Flag};
+use std::collections::HashMap;
+
+use crate::parser::Argument;
+use crate::parser::Parser;
+
+use super::parser::{Cmd, Flag};
 use super::ui::{Designation, Formatter, FormatterRules, Pattern, PredefinedThemes, Theme};
 use super::{Event, EventEmitter};
 
+type Callback = fn(HashMap<String, String>, HashMap<String, String>) -> ();
 /// The crux of the library, the program holds all information about your cli. It contains a vector field that stores all the commands that can be invoked from your program and also stores some metadata about your program
 pub struct Program {
     /// Stores all the commands that your program contains. You won't have to deal with this field directly rather by calling specific methods that allow you to build commands and add them to this vector
@@ -22,6 +28,12 @@ pub struct Program {
     /// A vector containing the flags/swicthed that can be passed to the root instance of the program and not the subcommands
     options: Vec<Flag>,
 
+    /// A vector holding args/params that can be passed to the program itself directly, rather than to its commands.
+    arguments: Vec<Argument>,
+
+    /// This is applicable in cases where the program can be executed directly without necessarily requiring a command to be passed to it
+    callback: Option<Callback>,
+
     /// An instance of the EventEmitter struct that the program can use to emit and listen to events. The program also contains utility functions to interface with the event_emitter which it contains.
     event_emitter: EventEmitter,
 
@@ -43,8 +55,10 @@ impl Program {
             about: "".to_owned(),
             theme: Theme::default(),
             author: "".to_owned(),
+            callback: None,
             pattern: Pattern::Legacy,
             version: "0.1.0".to_owned(),
+            arguments: vec![],
             event_emitter: EventEmitter::new(),
             options: vec![
                 Flag::new("-h --help", "Output help information for the program"),
@@ -129,8 +143,12 @@ impl Program {
         &self.options
     }
 
+    /// This method is similar to the `get_options` except it returns the params of the program, both required and optional ones
+    pub fn get_input(&self) -> &Vec<Argument> {
+        &self.arguments
+    }
+
     /// A private utility function that receives the first argument passed to the program, being the path to the binary file and extracts the name of the executable to be set as the name of the program and utilized when printing out help information.
-    ///
     /// The behavior of this function can be overriden by using the .bin_name() method. The method can be used when the name to be displayed to the users is different from the actual name of the executable binary.
     fn get_target_name(&self, val: String) -> String {
         if self.name.is_empty() {
@@ -148,55 +166,102 @@ impl Program {
         }
     }
 
+    pub fn argument(&mut self, val: &str, body: &str) -> &mut Program {
+        let arg = Argument::new(val, Some(body.to_string()));
+
+        self.arguments.push(arg);
+        self
+    }
+
+    /// A method for adding options/flags directly to the program. It follows the same rules as the Cmd.options() method
+    pub fn option(&mut self, body: &str, desc: &str) -> &mut Program {
+        let flag = Flag::new(body, desc);
+
+        if !self.options.contains(&flag) {
+            self.options.push(flag);
+        }
+
+        self
+    }
+
     /// This method receives the raw arguments passed to the program, and tries to get matches from all the configured commands or flags
     /// If no command is matched, it either acts in a default manner or executes the configured callbacks if any
     /// Also checks for the help and version flags.
+    fn _parse(&mut self, args: Vec<String>) {
+        if args.is_empty() {
+            let msg = if self.arguments.is_empty() && !self.cmds.is_empty() {
+                "You did not pass a command".to_string()
+            } else if !self.arguments.is_empty() && self.cmds.is_empty() {
+                let arg = self.arguments.first().unwrap();
+                format!("Missing required argument: {}", arg.literal)
+            } else {
+                "No command or arguments passed".to_string()
+            };
+            self.output_help(&msg);
+            self.emit(Event::OutputHelp, "");
+            return;
+        }
+
+        let first_arg = args[0].to_lowercase();
+        let parent = if self
+            .cmds
+            .iter()
+            .any(|c| c.get_name() == &first_arg || c.get_alias() == &first_arg)
+        {
+            "cmd"
+        } else {
+            "program"
+        };
+
+        match parent {
+            "cmd" => {
+                let cmd = self.get_cmd(&first_arg).unwrap();
+                let parser = Parser::new(self, Some(cmd));
+                let (values, options) = parser.parse(parent, &args[1..].to_vec());
+                (cmd.callback)(values, options);
+            }
+            _ => {
+                let parser = Parser::new(self, None);
+                let (vals, opts) = parser.parse(parent, &args);
+
+                if !self.arguments.is_empty() && self.callback.is_some() {
+                    (self.callback.unwrap())(vals, opts);
+                } else {
+                    self.emit(Event::UnknownCommand, &first_arg);
+                    let msg = format!("Unknown command \"{}\"", &first_arg);
+                    self.output_help(&msg);
+                    std::process::exit(1);
+                }
+            }
+        }
+    }
+
+    /// This method automatically receives the arguments passed to the program itself and parses the arguments accordingly.
     pub fn parse(&mut self) {
         let raw_args: Vec<String> = std::env::args().collect();
         let args = raw_args[1..].to_vec();
 
         self.name = self.get_target_name(raw_args[0].clone());
-
-        if args.is_empty() {
-            self.output_help("You did not pass a command!");
-            self.emit(Event::OutputHelp, "");
-            return;
-        }
-
-        match args[0].to_lowercase().as_str() {
-            val if self
-                .cmds
-                .iter()
-                .any(|c| c.get_name() == val || c.get_alias() == val) =>
-            {
-                let cmd = self.get_cmd(val).unwrap();
-                let (vals, opts) = cmd.parse(self, &args[1..].to_vec());
-                (cmd.callback)(vals, opts);
-            }
-            val if val.starts_with('-') => self.get_matches(val),
-            val => {
-                self.emit(Event::UnknownCommand, val);
-                let msg = format!("Unknown command \"{}\"", val);
-                self.output_help(msg.as_str());
-            }
-        }
+        self._parse(args);
     }
 
-    /// A method that try to get matches for any flags passed to the program itself, rather than a subcommand of the program.
-    fn get_matches(&self, val: &str) {
-        if let Some(v) = resolve_flag(&self.options, val) {
-            if v.short.as_str() == "-h" {
-                self.output_help("");
-                self.emit(Event::OutputHelp, "");
-            } else if v.short.as_str() == "-v" {
-                self.emit(Event::OutputVersion, self.version.as_str());
-                self.output_version_info()
-            }
-        } else {
-            self.emit(Event::UnknownOption, val);
-            let msg = format!("Unknown option \"{}\"", val);
-            self.output_help(msg.as_str());
+    /// Similar to the parse function with one fundamental difference. The parse function receives no arguments and will automatically get them from the args passed to the program. However, the parse from requires the args to parse to be passed to it as a vector of string slices.
+    pub fn parse_from(&mut self, values: Vec<&str>) {
+        let mut args = vec![];
+        for v in values {
+            args.push(v.to_string())
         }
+        self._parse(args)
+    }
+
+    /// Used to set the callback function that will be executed when the program is executed directly rather than a subcommand of the program.
+    pub fn action(
+        &mut self,
+        cb: fn(HashMap<String, String>, HashMap<String, String>) -> (),
+    ) -> &mut Program {
+        self.callback = Some(cb);
+
+        self
     }
 
     /// A simple method that tries to find a command from a given string slice that can either be the name of the command or its alias.
@@ -240,24 +305,57 @@ impl Program {
 
         use Designation::*;
 
-        fmtr.add(Description, &format!("\n{}\n", self.about));
+        if !self.about.is_empty() {
+            fmtr.add(Description, &format!("\n{}\n", self.about));
+        }
         fmtr.add(Headline, "\nUSAGE: \n");
         fmtr.add(Keyword, &format!("   {} ", self.name));
-        fmtr.add(Description, "<COMMAND> [options] \n");
+
+        let get_args = || {
+            let mut temp = String::new();
+            for arg in &self.arguments {
+                temp.push_str(&arg.literal);
+                temp.push(' ');
+            }
+            temp
+        };
+
+        if !self.cmds.is_empty() && !self.arguments.is_empty() {
+            let body = format!("[options] <COMMAND> | {} \n", get_args().trim());
+            fmtr.add(Description, &body);
+        } else if !self.cmds.is_empty() && self.arguments.is_empty() {
+            fmtr.add(Description, "[options] <COMMAND> \n")
+        } else {
+            fmtr.add(Description, &format!("[options] {} \n", get_args().trim()))
+        }
 
         fmtr.add(Headline, "\nOPTIONS: \n");
         fmtr.format(
             FormatterRules::Option(self.pattern.clone()),
             Some(self.options.clone()),
             None,
+            None,
         );
 
-        fmtr.add(Headline, "\nCOMMANDS: \n");
-        fmtr.format(
-            FormatterRules::Cmd(self.pattern.clone()),
-            None,
-            Some(self.cmds.clone()),
-        );
+        if !self.arguments.is_empty() {
+            fmtr.add(Headline, "\nARGS: \n");
+            fmtr.format(
+                FormatterRules::Args(self.pattern.clone()),
+                None,
+                None,
+                Some(self.arguments.clone()),
+            );
+        }
+
+        if !self.cmds.is_empty() {
+            fmtr.add(Headline, "\nCOMMANDS: \n");
+            fmtr.format(
+                FormatterRules::Cmd(self.pattern.clone()),
+                None,
+                Some(self.cmds.clone()),
+                None,
+            );
+        }
 
         if !err.is_empty() {
             fmtr.add(Error, &format!("\nError: {}\n", err))
@@ -299,13 +397,15 @@ mod test {
 
         let manual = Program {
             cmds: vec![],
-            options: vec![],
             name: "".to_owned(),
+            about: "a test".to_string(),
+            callback: None,
             theme: Theme::default(),
+            author: "me".to_string(),
+            options: vec![],
             pattern: Pattern::Legacy,
             version: "0.1.0".to_string(),
-            author: "me".to_string(),
-            about: "a test".to_string(),
+            arguments: vec![],
             event_emitter: EventEmitter::new(),
         };
 
